@@ -1,6 +1,6 @@
 ;;; forge-commands.el --- Commands                 -*- lexical-binding: t -*-
 
-;; Copyright (C) 2018-2021  Jonas Bernoulli
+;; Copyright (C) 2018-2022  Jonas Bernoulli
 
 ;; Author: Jonas Bernoulli <jonas@bernoul.li>
 ;; Maintainer: Jonas Bernoulli <jonas@bernoul.li>
@@ -43,6 +43,14 @@ for a repository using the command `forge-add-pullreq-refspec'."
                  (const :tag "Ask every time" ask)
                  (const :tag "Never add refspec" nil)))
 
+(defcustom forge-checkout-worktree-read-directory-function
+  'forge-checkout-worktree-default-read-directory-function
+  "Function used by `forge-checkout-worktree' to read worktree directory.
+Takes the pull-request as only argument and must return a directory."
+  :package-version '(forge . "0.4.0")
+  :group 'forge
+  :type 'function)
+
 ;;; Dispatch
 
 ;;;###autoload (autoload 'forge-dispatch "forge-commands" nil t)
@@ -57,7 +65,9 @@ for a repository using the command `forge-add-pullreq-refspec'."
     ("c p" "pull-request"  forge-create-pullreq)
     ("c u" "pull-request from issue" forge-create-pullreq-from-issue
      :if (lambda () (forge-github-repository-p (forge-get-repository nil))))
-    ("c f" "fork or remote" forge-fork)]
+    ("c f" "fork or remote" forge-fork)
+    """Merge"
+    (7 "M  " "merge using API" forge-merge)]
    ["List"
     ("l t" "topics"        forge-list-topics)
     ("l i" "issues"        forge-list-issues)
@@ -65,6 +75,8 @@ for a repository using the command `forge-add-pullreq-refspec'."
     ("l n" "notifications" forge-list-notifications)
     ("l r" "repositories"  forge-list-repositories)
     (7 "l a" "awaiting review"        forge-list-requested-reviews)
+    (7 "n i" "labeled issues"         forge-list-labeled-issues)
+    (7 "n p" "labeled pull-requests"  forge-list-labeled-pullreqs)
     (7 "m i" "authored issues"        forge-list-authored-issues)
     (7 "m p" "authored pull-requests" forge-list-authored-pullreqs)
     (7 "o i" "owned issues"           forge-list-owned-issues)
@@ -75,14 +87,16 @@ for a repository using the command `forge-add-pullreq-refspec'."
     ("v i" "issue"         forge-visit-issue)
     ("v p" "pull-request"  forge-visit-pullreq)
     """Browse"
-    ("b b" "issues"        forge-browse-issues)
+    ("b I" "issues"        forge-browse-issues)
+    ("b P" "pull-requests" forge-browse-pullreqs)
     ("b t" "topic"         forge-browse-topic)
     ("b i" "issue"         forge-browse-issue)
     ("b p" "pull-request"  forge-browse-pullreq)]]
   [["Configure"
-    ("a" "add repository to database" forge-add-repository)
-    ("r" "forge.repository" forge-forge.remote)
-    (7 "t" forge-toggle-display-in-status-buffer)]])
+    ("a  " "add repository to database" forge-add-repository)
+    ("r  " "forge.remote"  forge-forge.remote)
+    ("t t" forge-toggle-display-in-status-buffer)
+    ("t c" forge-toggle-closed-visibility)]])
 
 ;;; Pull
 
@@ -174,11 +188,14 @@ If pulling is too slow, then also consider setting the Git variable
   "Pull the API data for the current topic.
 If there is no current topic or with a prefix argument read a
 TOPIC to pull instead."
-  (interactive (list (forge-read-topic "Pull topic")))
-  (let* ((topic (forge-get-topic topic))
-         (repo (forge-get-repository topic)))
+  (interactive (list (forge-read-topic "Pull topic" nil t)))
+  (let ((repo (forge-get-repository t)))
     (forge--zap-repository-cache repo)
-    (forge--pull-topic repo topic)))
+    (forge--pull-topic repo
+                       (if (numberp topic)
+                           (forge-issue :repository (oref repo id)
+                                        :number topic)
+                         (forge-get-topic topic)))))
 
 (cl-defmethod forge--pull-topic ((repo forge-repository) _topic)
   (error "Fetching an individual topic not implemented for %s"
@@ -257,6 +274,15 @@ Prefer a topic over a branch and that over a commit."
   (interactive (list (magit-read-remote "Browse remote")))
   (browse-url (forge--format remote 'remote-url-format)))
 
+;;;###autoload
+(defun forge-browse-repository (repo)
+  "View the current repository in a separate buffer."
+  (interactive
+   (list (or (forge-current-repository)
+             (forge-get-repository
+              (forge-read-repository "Browse repository")))))
+  (browse-url (forge--format repo 'remote-url-format)))
+
 ;;;###autoload
 (defun forge-browse-topic ()
   "Visit the current topic using a browser."
@@ -721,21 +747,10 @@ This is like `forge-checkout-pullreq', except that it also
 creates a new worktree. Please see the manual for more
 information."
   (interactive
-   (let* ((id (forge-read-pullreq "Checkout pull request" t))
-          (pullreq (forge-get-pullreq id)))
-     (with-slots (number head-ref) pullreq
-       (let ((path (read-directory-name
-                    (format "Checkout #%s in new worktree: " number)
-                    (file-name-directory
-                     (directory-file-name default-directory))
-                    nil nil
-                    (let ((branch (forge--pullreq-branch-internal pullreq)))
-                      (if (string-match-p "\\`pr-[0-9]+\\'" branch)
-                          (number-to-string number)
-                        (format "%s-%s" number head-ref))))))
-         (when (equal path "")
-           (user-error "The empty string isn't a valid path"))
-         (list path id)))))
+   (let ((id (forge-read-pullreq "Checkout pull request" t)))
+     (list (funcall forge-checkout-worktree-read-directory-function
+                    (forge-get-pullreq id))
+           id)))
   (when (and (file-exists-p path)
              (not (and (file-directory-p path)
                        (= (length (directory-files "/tmp/testing/")) 2))))
@@ -745,6 +760,22 @@ information."
                              (forge-branch-pullreq
                               (forge-get-pullreq pullreq)))))
 
+(defun forge-checkout-worktree-default-read-directory-function (pullreq)
+  (with-slots (number head-ref) pullreq
+    (let ((path (read-directory-name
+                 (format "Checkout #%s in new worktree: " number)
+                 (file-name-directory
+                  (directory-file-name default-directory))
+                 nil nil
+                 (let ((branch (forge--pullreq-branch-internal pullreq)))
+                   (if (string-match-p "\\`pr-[0-9]+\\'" branch)
+                       (number-to-string number)
+                     (format "%s-%s" number
+                             (replace-regexp-in-string "/" "-" head-ref)))))))
+      (when (equal path "")
+        (user-error "The empty string isn't a valid path"))
+      path)))
+
 ;;; Marks
 
 (defun forge-create-mark (name face description)
@@ -841,6 +872,7 @@ is added anyway.  Currently this only supports Github and Gitlab."
 ;;; Misc
 
 (transient-define-infix forge-forge.remote ()
+  "Change the local value of the `forge.remote' Git variable."
   :class 'magit--git-variable:choices
   :variable "forge.remote"
   :choices 'magit-list-remotes
@@ -853,17 +885,24 @@ is added anyway.  Currently this only supports Github and Gitlab."
   (forge-notifications-setup-buffer))
 
 (transient-define-suffix forge-toggle-display-in-status-buffer ()
+  "Toggle whether to display topics in the current status buffer."
   :description (lambda ()
                  (if forge-display-in-status-buffer
-                     "hide topics"
+                     "hide all topics"
                    "display topics"))
-  "Toggle whether to display topics in the current Magit status buffer."
   (interactive)
   (setq forge-display-in-status-buffer (not forge-display-in-status-buffer))
   (magit-refresh))
 
-(defun forge-toggle-closed-visibility ()
-  "Toggle whether recently closed issues are shown."
+(transient-define-suffix forge-toggle-closed-visibility ()
+  "Toggle whether to display recently closed topics.
+This only affect the current status buffer."
+  :description (lambda ()
+                 (if (or (atom forge-topic-list-limit)
+                         (> (cdr forge-topic-list-limit) 0))
+                     "hide closed topics"
+                   "display recently closed topics"))
+  :inapt-if-not (lambda () forge-display-in-status-buffer)
   (interactive)
   (magit-repository-local-delete (list 'forge-ls-recent-topics 'issue))
   (magit-repository-local-delete (list 'forge-ls-recent-topics 'pullreq))
