@@ -1,6 +1,6 @@
 ;;; forge-list.el --- Tabulated-list interface     -*- lexical-binding: t -*-
 
-;; Copyright (C) 2018-2021  Jonas Bernoulli
+;; Copyright (C) 2018-2022  Jonas Bernoulli
 
 ;; Author: Jonas Bernoulli <jonas@bernoul.li>
 ;; Maintainer: Jonas Bernoulli <jonas@bernoul.li>
@@ -239,8 +239,8 @@ This is a list of package names.  Used by the commands
   "List issues of the current repository that have LABEL.
 List them in a separate buffer."
   (interactive (list (oref (forge-get-repository t) id)
-                     (magit-completing-read-multiple*
-                      "Label: "
+                     (magit-completing-read
+                      "Label"
                       (mapcar #'cadr (oref (forge-get-repository t) labels))
                       nil t)))
   (forge-topic-list-setup #'forge-issue-list-mode id nil nil
@@ -311,8 +311,8 @@ Only Github is supported for now."
   "List pull-requests of the current repository that have LABEL.
 List them in a separate buffer."
   (interactive (list (oref (forge-get-repository t) id)
-                     (magit-completing-read-multiple*
-                      "Label: "
+                     (magit-completing-read
+                      "Label"
                       (mapcar #'cadr (oref (forge-get-repository t) labels))
                       nil t)))
   (forge-topic-list-setup #'forge-pullreq-list-mode id nil nil
