@@ -1,6 +1,6 @@
 ;;; forge-topic.el --- Topics support              -*- lexical-binding: t -*-
 
-;; Copyright (C) 2018-2021  Jonas Bernoulli
+;; Copyright (C) 2018-2022  Jonas Bernoulli
 
 ;; Author: Jonas Bernoulli <jonas@bernoul.li>
 ;; Maintainer: Jonas Bernoulli <jonas@bernoul.li>
@@ -100,6 +100,11 @@ This variable has to be customized before `forge' is loaded."
   "Whether to display topics in the current Magit status buffer.")
 (put 'forge-display-in-status-buffer 'permanent-local t)
 
+(defvar forge-format-avatar-function nil
+  "Function used to insert avatars in certain locations.
+This is experimental and intended for users who wish to
+implement such a function themselves.  See #447.")
+
 ;;; Faces
 
 (defface forge-topic-unread
@@ -307,9 +312,10 @@ This variable has to be customized before `forge' is loaded."
              (setq topic-section-type 'pullreq)))
       (magit-insert-section ((eval list-section-type) nil t)
         (magit-insert-heading
-          (format "%s (%s)"
-                  (propertize heading 'font-lock-face 'magit-section-heading)
-                  (length topics)))
+          (concat (magit--propertize-face (concat heading " ")
+                                          'magit-section-heading)
+                  (magit--propertize-face (format "(%s)" (length topics))
+                                          'magit-section-child-count)))
         (magit-make-margin-overlay nil t)
         (magit-insert-section-body
           (dolist (topic topics)
@@ -360,6 +366,8 @@ identifier."
     (define-key map (kbd "C-c C-n") 'forge-create-post)
     (define-key map (kbd "C-c C-r") 'forge-create-post)
     (define-key map [remap magit-browse-thing] 'forge-browse-topic)
+    (define-key map [remap magit-visit-thing] 'markdown-follow-link-at-point)
+    (define-key map [mouse-2] 'markdown-follow-link-at-point)
     map))
 
 (define-derived-mode forge-topic-mode magit-mode "View Topic"
@@ -431,7 +439,8 @@ identifier."
             (let ((heading
                    (format-spec
                     forge-post-heading-format
-                    `((?a . ,(propertize (or author "(ghost)")
+                    `((?a . ,(propertize (concat (forge--format-avatar author)
+                                                 (or author "(ghost)"))
                                          'font-lock-face 'forge-post-author))
                       (?c . ,(propertize created 'font-lock-face 'forge-post-date))
                       (?C . ,(propertize (apply #'format "%s %s ago"
@@ -469,9 +478,16 @@ identifier."
 (cl-defun forge-insert-topic-state
     (&optional (topic forge-buffer-topic))
   (magit-insert-section (topic-state)
-    (insert (format "%-11s" "State: ")
-            (symbol-name (oref topic state))
-            "\n")))
+    (insert (format
+             "%-11s%s\n" "State: "
+             (let ((state (oref topic state)))
+               (magit--propertize-face
+                (symbol-name state)
+                (pcase (list state (forge-pullreq-p (forge-topic-at-point)))
+                  (`(merged) 'forge-topic-merged)
+                  (`(closed) 'forge-topic-closed)
+                  (`(open t) 'forge-topic-unmerged)
+                  (`(open)   'forge-topic-open))))))))
 
 (defvar forge-topic-milestone-section-map
   (let ((map (make-sparse-keymap)))
@@ -617,7 +633,9 @@ Return a value between 0 and 1."
     (insert (format "%-11s" "Assignees: "))
     (if-let ((assignees (closql--iref topic 'assignees)))
         (insert (mapconcat (pcase-lambda (`(,login ,name))
-                             (format "%s (@%s)" name login))
+                             (format "%s%s (@%s)"
+                                     (forge--format-avatar login)
+                                     name login))
                            assignees ", "))
       (insert (propertize "none" 'font-lock-face 'magit-dimmed)))
     (insert ?\n)))
@@ -635,7 +653,9 @@ Return a value between 0 and 1."
       (insert (format "%-11s" "Review-Requests: "))
       (if-let ((review-requests (closql--iref topic 'review-requests)))
           (insert (mapconcat (pcase-lambda (`(,login ,name))
-                               (format "%s (@%s)" name login))
+                               (format "%s%s (@%s)"
+                                       (forge--format-avatar login)
+                                       name login))
                              review-requests ", "))
         (insert (propertize "none" 'font-lock-face 'magit-dimmed)))
       (insert ?\n))))
@@ -666,9 +686,14 @@ Return a value between 0 and 1."
               (and (not (string-prefix-p "/" file)) "/")
               file))))
 
+(defun forge--format-avatar (author)
+  (if forge-format-avatar-function
+      (funcall forge-format-avatar-function author)
+    ""))
+
 ;;; Completion
 
-(defun forge-read-topic (prompt &optional type)
+(defun forge-read-topic (prompt &optional type allow-number)
   (when (eq type t)
     (setq type (if current-prefix-arg nil 'open)))
   (let* ((default (forge-current-topic))
@@ -679,14 +704,19 @@ Return a value between 0 and 1."
                     (nconc
                      (forge-ls-pullreqs repo type [number title id class])
                      (forge-ls-issues   repo type [number title id class]))
-                    #'> :key #'car))))
-    (cdr (assoc (magit-completing-read
-                 prompt choices nil nil nil nil
-                 (and default
-                      (setq default (forge--topic-format-choice default))
-                      (member default choices)
-                      (car default)))
-                choices))))
+                    #'> :key #'car)))
+         (choice  (magit-completing-read
+                   prompt choices nil nil nil nil
+                   (and default
+                        (setq default (forge--topic-format-choice default))
+                        (member default choices)
+                        (car default)))))
+    (or (cdr (assoc choice choices))
+        (and allow-number
+             (let ((number (string-to-number choice)))
+               (if (= number 0)
+                   (user-error "Not an existing topic or number: %s")
+                 number))))))
 
 (cl-defmethod forge--topic-format-choice ((topic forge-topic))
   (cons (format "%s%s  %s"
@@ -917,12 +947,12 @@ modify `bug-reference-bug-regexp' if appropriate."
           (setq-local bug-reference-bug-regexp
                       (if (forge--childp repo 'forge-gitlab-repository)
                           "\\(?3:[!#]\\)\\(?2:[0-9]+\\)"
-                        "#\\(?2:[0-9]+\\)"))))
-      (if (derived-mode-p 'prog-mode)
-          (bug-reference-prog-mode 1)
-        (bug-reference-mode 1))
-      (add-hook 'completion-at-point-functions
-                'forge-topic-completion-at-point nil t))))
+                        "#\\(?2:[0-9]+\\)")))
+        (if (derived-mode-p 'prog-mode)
+            (bug-reference-prog-mode 1)
+          (bug-reference-mode 1))
+        (add-hook 'completion-at-point-functions
+                  'forge-topic-completion-at-point nil t)))))
 
 (when (and (not noninteractive) forge--sqlite-available-p)
   (dolist (hook forge-bug-reference-hooks)
